import click
import yaml
import re
from kubernetes import client
from os import path
from utils import hiss, util
from settings import settings

def gen_channel_artifact():

    domains = settings.ORDERER_DOMAINS.split(' ')

    # Create temp folder & namespace
    settings.k8s.prereqs(domains[0])

    k8s_template_file = '%s/channel-artifacts/fabric-deployment-channel-artifacts.yaml' % util.get_k8s_template_path()
    dict_env = {
        'ORDERER_DOMAIN': domains[0],
        'FABRIC_TAG': settings.FABRIC_TAG,
        'EFS_SERVER': settings.EFS_SERVER,
        'EFS_PATH': settings.EFS_PATH,
        'EFS_EXTEND': settings.EFS_EXTEND,
        'PVS_PATH': settings.PVS_PATH
    }

    settings.k8s.apply_yaml_from_template(
        namespace=domains[0], k8s_template_file=k8s_template_file, dict_env=dict_env)

def del_gen_channel_artifact():

    domains = settings.ORDERER_DOMAINS.split(' ')
    jobname = 'channel-artifacts'

    # Delete job pod
    return settings.k8s.delete_job(name=jobname, namespace=domains[0])
    

@click.group()
def channel_artifact():
    """Generate channel.tx, genesis.block"""
    pass

@channel_artifact.command('setup', short_help="Run job to generate channel.tx, genesis.block")
def setup():
    hiss.rattle('Generate channel.tx, genesis.block')

    gen_channel_artifact()

@channel_artifact.command('delete', short_help="Delete job channel artifact")
def delete():
    hiss.rattle('Delete job channel artifact')

    del_gen_channel_artifact()



import click
from kubernetes import client
from os import path
from utils import hiss, util
from settings import settings

def deploy_external_cc(peer, cc_name, cc_image, cc_package_id):
    # Get domain
    domain = util.get_domain(peer)
    
    # Create temp folder & namespace
    settings.k8s.prereqs(domain)

    # Create config map
    k8s_template_file = '%s/external-chaincode/chaincode-stateful.yaml' % util.get_k8s_template_path()
    dict_env = {
        'PEER_NAME': peer,
        'PEER_DOMAIN': domain,
        'CHAINCODE_NAME': cc_name,
        'CHAINCODE_IMAGE': cc_image,
        'CHAINCODE_PACKAGE_ID': cc_package_id
    }
    settings.k8s.apply_yaml_from_template(
        namespace=domain, k8s_template_file=k8s_template_file, dict_env=dict_env)

    chaincode_service = '%s/external-chaincode/chaincode-service.yaml' % util.get_k8s_template_path()
    settings.k8s.apply_yaml_from_template(
        namespace=domain, k8s_template_file=chaincode_service, dict_env=dict_env)

def delete_external_cc(peer):
    ## Find explorer_db pod
    domain = util.get_domain(peer)
    list_stateful_set = settings.k8s.find_stateful_set(namespace=domain, keyword="chaincode")
    # Remove chaincode container of another org
    for sts in list_stateful_set:
        if peer not in sts: list_stateful_set.remove(sts)
    for sts in list_stateful_set:
        # Delete stateful set
        return settings.k8s.delete_stateful(name=sts, namespace=domain)

def config_peer(peer):
    # Get domain
    domain = util.get_domain(peer)

    # Create temp folder & namespace
    settings.k8s.prereqs(domain)

    # Create config map
    k8s_template_file = '%s/external-chaincode/builders-config.yaml' % util.get_k8s_template_path()
    dict_env = {
        'PEER_DOMAIN': domain
    }
    settings.k8s.apply_yaml_from_template(
        namespace=domain, k8s_template_file=k8s_template_file, dict_env=dict_env)

def deploy_all_external_cc(ccname, ccimage, packageid):
    orgs = settings.PEER_ORGS.split(' ')
    # TODO: Multiprocess
    for org in orgs:
        deploy_external_cc(org, ccname, ccimage, packageid)

def config_all_peer():
    orgs = settings.PEER_ORGS.split(' ')
    # TODO: Multiprocess
    for org in orgs:
        config_peer(org)

def del_config():
    print('TODO')

def delete_all_external_cc():

    peers = settings.PEER_ORGS.split(' ')
    results = []
    for peer in peers:
        domain = util.get_domain(peer)
        results.append(delete_external_cc(peer))
    return results


@click.group()

def externalCC():
    """External Chaincode"""
    pass

@externalCC.command('config', short_help="Create config map")
def config():
    hiss.rattle('Create config map')
    config_all_peer()

@externalCC.command('deploy', short_help="Deploy external chaincode")
@click.option('--ccname', help="Chaincode name")
@click.option('--ccimage', help="Chaincode image")
@click.option('--packageid', help="Chaincode package Id")
def deploy(ccname, ccimage, packageid):
    hiss.rattle('Deploy external chaincode')
    deploy_all_external_cc(ccname, ccimage, packageid)

@externalCC.command('delete', short_help="Delete external chaincode")
def delete():
    hiss.rattle('Delete external chaincode')
    delete_all_external_cc()

@externalCC.command('delConfig', short_help="Delete config map")
def delete():
    hiss.rattle('Delete config map')
    del_config()



import click
import yaml
import re
import datetime
from kubernetes import client
from os import path
from kubernetes.client.rest import ApiException
from utils import hiss, util
from settings import settings


def terminate_kafka():
    domain = settings.KAFKA_NAMESPACE
    name = 'kafka'

    # Terminate stateful set
    return settings.k8s.delete_stateful(name=name, namespace=domain, delete_pvc=True)

def delete_kafka():
    domain = settings.KAFKA_NAMESPACE
    name = 'kafka'

    # Delete stateful set
    return settings.k8s.delete_stateful(name=name, namespace=domain)

def setup_kafka():

    domain = settings.KAFKA_NAMESPACE

    # Create temp folder & namespace
    settings.k8s.prereqs(domain)

    dict_env = {
        'KAFKA_NAMESPACE': domain
    }

    zk_cs_template_file = '%s/kafka/0kafka-hs.yaml' % util.get_k8s_template_path()
    settings.k8s.apply_yaml_from_template(
        namespace=domain, k8s_template_file=zk_cs_template_file, dict_env=dict_env)

    zk_hs_template_file = '%s/kafka/1kafka-cs.yaml' % util.get_k8s_template_path()
    settings.k8s.apply_yaml_from_template(
        namespace=domain, k8s_template_file=zk_hs_template_file, dict_env=dict_env)

    zk_set_template_file = '%s/kafka/2kafka-set.yaml' % util.get_k8s_template_path()
    settings.k8s.apply_yaml_from_template(
        namespace=domain, k8s_template_file=zk_set_template_file, dict_env=dict_env)


@click.group()
def kafka():
    """Kafka"""
    pass


@kafka.command('setup', short_help="Setup Kafka")
def setup():
    hiss.rattle('Setup Kafka')
    setup_kafka()

@kafka.command('delete', short_help="Delete Kafka")
def delete():
    hiss.rattle('Delete Kafka')
    delete_kafka()

@kafka.command('terminate', short_help="Terminate Kafka")
def terminate():
    hiss.rattle('Terminate Kafka')
    terminate_kafka()



import re
import yaml
import json
import datetime
from kubernetes import client
from kubernetes.client.rest import ApiException
from os import path

from settings import settings
from utils import util, hiss


class Namespace:
    def __init__(self, ns_name):
        self.name = ns_name

    def create(self):
        # Load template file
        k8s_template_file = '%s/namespace/namespaces.yaml' % util.get_k8s_template_path()
        with open(k8s_template_file, 'r') as sources:
            lines = sources.readlines()
            out_data = []

        # Replace variable
        for line in lines:
            out_line = re.sub(r'{{NAMESPACES}}', self.name, line)
            out_data.append(out_line)

        # Get current datetime (UTC)
        current_time = datetime.datetime.utcnow().replace(
            microsecond=0).isoformat().split('T')

        # Make folder temp if it not exists
        tmp_path = "%s/%s" % (util.get_temp_path(), current_time[0])
        util.make_folder(tmp_path)

        # Create yaml_path
        yaml_path = '%s/%s/%s_namespaces.yaml' % (util.get_temp_path(), current_time[0], current_time[1])
        # Write yaml -> yaml_path
        with open(yaml_path, "w") as sources:
            for line in out_data:
                sources.write(line)

        # Execute yaml
        with open(yaml_path) as f:
            dep = yaml.safe_load(f)
            try:
                settings.k8s.coreApi.create_namespace(body=dep)
                hiss.sub_echo('Create namespace successfully')
                return True
            except ApiException as e:
                return hiss.hiss("Exception when calling CoreV1Api->create_namespace: %s\n" % e)

    def get(self):
        try:
            list = settings.k8s.coreApi.list_namespace()
        except ApiException as e:
            return hiss.hiss("Exception when calling CoreV1Api->list_namespace: %s\n" % e)
        ns = next((x for x in list.items if x.metadata.name == self.name), None)
        if not ns:
            return False
        else:
            return ns

    def delete(self):
        try:
            api_response = settings.k8s.coreApi.delete_namespace(name=self.name)
            return api_response
        except ApiException as e:
            return hiss.hiss("Exception when calling CoreV1Api->delete_namespace: %s\n" % e)



import subprocess
import re
from os.path import expanduser
import os
import shutil
import click
from dotenv import load_dotenv
from utils import util, hiss
from utils.kube import KubeHelper

DEFAULT_CONFIG_PATH = expanduser('~/.akachain/akc-mamba/mamba/config/.env')
DEFAULT_SCRIPT_PATH = expanduser('~/.akachain/akc-mamba/mamba/scripts')
DEFAULT_TEMPLATE_PATH = expanduser('~/.akachain/akc-mamba/mamba/template')
DEFAULT_OTHER_PATH = expanduser('~/.akachain/akc-mamba/mamba/k8s/')


def get_template_env():
    return util.get_package_resource('config', 'operator.env-template')

def detect_deployed_efs_server(k8s_type):
    hiss.echo('Looking for deployed efs server...')
    efs_server = ''
    if k8s_type == 'eks':
        # Find efs-efs-provisioner pod
        pods = KubeHelper().find_pod(namespace="default", keyword="efs-efs-provisioner")
        if not pods:
            return efs_server
        efs_server_cmd = 'kubectl describe deployments -n default efs-efs-provisioner | grep Server'
        detected_efs_server = subprocess.check_output(
            efs_server_cmd, shell=True)
        if detected_efs_server:
            efs_server = detected_efs_server.decode().split(':')[1].strip()
    elif k8s_type == 'minikube':
        efs_server_cmd = 'kubectl get svc -n default | grep nfs-server | awk \'{print $3}\''
        detected_efs_server = subprocess.check_output(
            efs_server_cmd, shell=True)
        if detected_efs_server:
            efs_server = detected_efs_server.decode().strip()
    return efs_server

def detect_deployed_efs_path():
    hiss.echo('Looking for deployed efs path...')
    efs_path = ''
    efs_path_cmd = 'kubectl get pvc | grep efs | awk \'{print $3}\''
    detected_efs_path = subprocess.check_output(efs_path_cmd, shell=True)
    if detected_efs_path:
        efs_path = detected_efs_path.decode().strip()
    return efs_path

def detect_deployed_efs_pod():
    hiss.echo('Looking for deployed efs pod...')
    efs_pod = ''
    efs_pod_cmd = 'kubectl get pod -n default | grep test-efs | awk \'{print $1}\''
    detected_efs_pod = subprocess.check_output(efs_pod_cmd, shell=True)
    if detected_efs_pod:
        efs_pod = detected_efs_pod.decode().strip()
    return efs_pod

def extract_cfg(mamba_config, dev_mode):
    hiss.echo('Extract config to default config path: %s ' %
              DEFAULT_CONFIG_PATH)
    dotenv_path = get_template_env()
    if not os.path.isdir(mamba_config):
        os.makedirs(mamba_config)
    shutil.copy(dotenv_path, DEFAULT_CONFIG_PATH)
    load_dotenv(DEFAULT_CONFIG_PATH)

    default_cluster_name = os.getenv('EKS_CLUSTER_NAME')
    default_k8s_type = os.getenv('K8S_TYPE')
    deployment_mode = os.getenv('DEPLOYMENT_ENV')

    # Input
    cluster_name = input(
        f'Cluster name ({default_cluster_name}): ') or default_cluster_name
    k8s_type = input(
        f'Kubenetes type - support eks or minikube ({default_k8s_type}): ') or default_k8s_type
    if dev_mode:
        deployment_mode = 'develop'

    # Detect current environment setting
    # EFS_SERVER
    efs_server = detect_deployed_efs_server(k8s_type)
    # EFS_SERVER_ID
    efs_server_id = efs_server.split('.')[0]
    print('efs_server: ', efs_server)
    # EFS_PATH
    efs_path = detect_deployed_efs_path()
    # EFS_POD
    efs_pod = detect_deployed_efs_pod()

    if not efs_pod and k8s_type == 'eks':
        retry = 3
        while retry > 0:
            efs_server = input('EFS server (*):')
            if efs_server:
                break
            retry -= 1
        if not efs_server:
            hiss.hiss('Must specify EFS_SERVER!')
            exit()

    with open(DEFAULT_CONFIG_PATH, "r") as sources:
        lines = sources.readlines()
    with open(DEFAULT_CONFIG_PATH, "w") as sources:
        for line in lines:
            newline = re.sub(r'EKS_CLUSTER_NAME=.*',
                             f'EKS_CLUSTER_NAME=\"{cluster_name}\"', line)
            newline = re.sub(
                r'K8S_TYPE=.*', f'K8S_TYPE=\"{k8s_type}\"', newline)

            if efs_server:
                newline = re.sub(r'EFS_SERVER=.*',
                                 f'EFS_SERVER=\"{efs_server}\"', newline)
            if efs_server_id:
                newline = re.sub(r'EFS_SERVER_ID=.*',
                                 f'EFS_SERVER_ID=\"{efs_server_id}\"', newline)
            if efs_path:
                newline = re.sub(
                    r'EFS_PATH=.*', f'EFS_PATH=\"efs-{efs_path}\"', newline)
            if efs_pod:
                newline = re.sub(
                    r'EFS_POD=.*', f'EFS_POD=\"{efs_pod}\"', newline)
            newline = re.sub(
                    r'DEPLOYMENT_ENV=.*', f'DEPLOYMENT_ENV=\"{deployment_mode}\"', newline)
            sources.write(newline)

    hiss.rattle("See more config in %s" % DEFAULT_CONFIG_PATH)


def extract(force_all, force_config, force_script, force_template, force_other, dev_mode):
    # Extract config
    mamba_config = os.path.dirname(DEFAULT_CONFIG_PATH)

    if not os.path.isdir(mamba_config) or force_all or force_config:
        extract_cfg(mamba_config, dev_mode)

    # Extract scripts
    if not os.path.isdir(DEFAULT_SCRIPT_PATH) or force_all or force_script:
        hiss.echo('Extract scripts to default scripts path: %s ' %
                  DEFAULT_SCRIPT_PATH)
        script_path = util.get_package_resource('', 'scripts')
        if os.path.isdir(DEFAULT_SCRIPT_PATH):
            shutil.rmtree(DEFAULT_SCRIPT_PATH)
        shutil.copytree(script_path, DEFAULT_SCRIPT_PATH)

    # Extract template
    if not os.path.isdir(DEFAULT_TEMPLATE_PATH) or force_all or force_template:
        hiss.echo('Extract template to default template path: %s ' %
                  DEFAULT_TEMPLATE_PATH)
        template_path = util.get_package_resource('', 'template')
        if os.path.isdir(DEFAULT_TEMPLATE_PATH):
            shutil.rmtree(DEFAULT_TEMPLATE_PATH)
        shutil.copytree(template_path, DEFAULT_TEMPLATE_PATH)

    # Extract other
    if not os.path.isdir(DEFAULT_OTHER_PATH) or force_all or force_other:
        hiss.echo('Extract other to default other path: %s ' %
                  DEFAULT_OTHER_PATH)
        other_path = util.get_package_resource('k8s', '')
        if os.path.isdir(DEFAULT_OTHER_PATH):
            shutil.rmtree(DEFAULT_OTHER_PATH)
        shutil.copytree(other_path, DEFAULT_OTHER_PATH)


@click.command('init', short_help="Extract binary config")
@click.option('-f', '--force', is_flag=True, help="Force extract all")
@click.option('-c', '--config', is_flag=True, help="Force extract config")
@click.option('-s', '--script', is_flag=True, help="Force extract script")
@click.option('-t', '--template', is_flag=True, help="Force extract template")
@click.option('-o', '--other', is_flag=True, help="Force extract other")
@click.option('--dev', is_flag=True, help="Develop mode")
def extract_config(force, config, script, template, other, dev):
    extract(force, config, script, template, other, dev)



import click
import os
import time
from settings import settings
from os import path

from utils import hiss, util

def get_working_orgs_domain():
    orgs = settings.PEER_ORGS.split(' ')
    if len(orgs) > 1:
        hiss.hiss('Function is still develop')
    domain = util.get_domain(orgs[0])
    settings.k8s.prereqs(domain)

    return [orgs[0], domain]

def fetch_config(org=None, domain=None):

    if not org or not domain:
        [org, domain] = get_working_orgs_domain()

    dict_env = {
        'ORG_NAME': org,
        'ORG_DOMAIN': domain,
        'ORDERER_NAME': settings.ORDERER_ORGS,
        'ORDERER_DOMAIN': settings.ORDERER_DOMAINS,
        'CHANNEL_NAME': settings.CHANNEL_NAME,
        'FABRIC_TAG': settings.FABRIC_TAG,
        'EFS_SERVER': settings.EFS_SERVER,
        'EFS_PATH': settings.EFS_PATH,
        'EFS_EXTEND': settings.EFS_EXTEND,
        'PVS_PATH': settings.PVS_PATH
    }
    k8s_template_file = '%s/add-org/2fetch-channel.yaml' % util.get_k8s_template_path()
    settings.k8s.apply_yaml_from_template(
        namespace=domain, k8s_template_file=k8s_template_file, dict_env=dict_env)

def modify_config(domain=None):
    if not domain:
        [_, domain] = get_working_orgs_domain()

    dict_env = {
        'ORG_DOMAIN': domain,
        'NEW_ORG_NAME': settings.NEW_ORG_NAME,
        'FABRIC_TAG': settings.FABRIC_TAG,
        'EFS_SERVER': settings.EFS_SERVER,
        'EFS_PATH': settings.EFS_PATH,
        'EFS_EXTEND': settings.EFS_EXTEND,
        'PVS_PATH': settings.PVS_PATH
    }
    k8s_template_file = '%s/add-org/3modifyingorgmaterial.yaml' % util.get_k8s_template_path()
    settings.k8s.apply_yaml_from_template(
        namespace=domain, k8s_template_file=k8s_template_file, dict_env=dict_env)

def create_config_update_pb(domain=None):

    if not domain:
        [_, domain] = get_working_orgs_domain()

    dict_env = {
        'ORG_DOMAIN': domain,
        'CHANNEL_NAME': settings.CHANNEL_NAME,
        'FABRIC_TAG': settings.FABRIC_TAG,
        'EFS_SERVER': settings.EFS_SERVER,
        'EFS_PATH': settings.EFS_PATH,
        'EFS_EXTEND': settings.EFS_EXTEND,
        'PVS_PATH': settings.PVS_PATH
    }
    k8s_template_file = '%s/add-org/4createconfigupdate.yaml' % util.get_k8s_template_path()
    settings.k8s.apply_yaml_from_template(
        namespace=domain, k8s_template_file=k8s_template_file, dict_env=dict_env)

def update_channel_config(org=None, domain=None):

    if not org or not domain:
        [org, domain] = get_working_orgs_domain()

    dict_env = {
        'ORG_DOMAIN': domain,
        'ORG_NAME': org,
        'CHANNEL_NAME': settings.CHANNEL_NAME,
        'ORDERER_NAME': settings.ORDERER_ORGS,
        'ORDERER_DOMAIN': settings.ORDERER_DOMAINS,
        'FABRIC_TAG': settings.FABRIC_TAG,
        'EFS_SERVER': settings.EFS_SERVER,
        'EFS_PATH': settings.EFS_PATH,
        'EFS_EXTEND': settings.EFS_EXTEND,
        'PVS_PATH': settings.PVS_PATH
    }
    k8s_template_file = '%s/add-org/6updatechannelconfig.yaml' % util.get_k8s_template_path()
    settings.k8s.apply_yaml_from_template(
        namespace=domain, k8s_template_file=k8s_template_file, dict_env=dict_env)


def auto_update_channel_config():

    fetch_config()
    modify_config()
    create_config_update_pb()
    time.sleep(2)
    update_channel_config()
    return True

def del_fetch_config(domain=None):
    if not domain:
        [_, domain] = get_working_orgs_domain()
    jobname = 'fetch-channel'
    # Delete job pod
    return settings.k8s.delete_job(name=jobname, namespace=domain)

def del_modify_config(domain=None):
    if not domain:
        [_, domain] = get_working_orgs_domain()
    jobname = 'modifyingorgmaterial'
    # Delete job pod
    return settings.k8s.delete_job(name=jobname, namespace=domain)

def del_config_update_pb(domain=None):
    if not domain:
        [_, domain] = get_working_orgs_domain()
    jobname = 'createconfigupdate'
    # Delete job pod
    return settings.k8s.delete_job(name=jobname, namespace=domain)

def del_update_channel_config(domain=None):
    if not domain:
        [_, domain] = get_working_orgs_domain()
    jobname = 'updatechannelconfig'
    # Delete job pod
    return settings.k8s.delete_job(name=jobname, namespace=domain)

def auto_del_update_channel_config():
    del_fetch_config()
    del_modify_config()
    del_config_update_pb()
    del_update_channel_config()

@click.group()
def channel_config():
    """Channel config"""
    pass

@channel_config.command('auto-update', short_help="Create all job update channel config")
def auto_update():
    hiss.rattle('Update channel config')
    auto_update_channel_config()

@channel_config.command('auto-delete', short_help="Delete all job update channel config")
def auto_delete():
    hiss.rattle('Delete channel config')
    auto_del_update_channel_config()

@channel_config.command('fetch', short_help="Fetch channel config")
def fetch():
    hiss.rattle('Fetch channel config')
    fetch_config()

@channel_config.command('create-pb', short_help="Create modify config pb")
def create_pb():
    hiss.rattle('Create modify config pb')
    create_config_update_pb()

@channel_config.command('update', short_help="Update channel config using modify config pb")
def update():
    hiss.rattle('Update channel config using modify config pb')
    update_channel_config()

@channel_config.command('del-fetch', short_help="Delete job fetch channel config")
def del_fetch():
    hiss.rattle('Delete job fetch channel config')
    del_fetch_config()

@channel_config.command('del-create-pb', short_help="Delete job create modify config pb")
def del_create_pb():
    hiss.rattle('Delete job create modify config pb')
    del_config_update_pb()

@channel_config.command('del-update', short_help="Delete job update channel config using modify config pb")
def del_update():
    hiss.rattle('Delete job update channel config using modify config pb')
    del_update_channel_config()



import click
import os
import time
from settings import settings
from os import path

from utils import hiss, util

def update_anchor_peer(org):

    # Get domain
    domain = util.get_domain(org)
    # Create temp folder & namespace
    settings.k8s.prereqs(domain)

    dict_env = {
        'ORG_NAME': org,
        'ORG_DOMAIN': domain,
        'ORGS': settings.PEER_ORGS,
        'ORDERER_NAME': settings.ORDERER_ORGS,
        'ORDERER_DOMAIN': settings.ORDERER_DOMAINS,
        'CHANNEL_NAME': settings.CHANNEL_NAME,
        'FABRIC_TAG': settings.FABRIC_TAG,
        'EFS_SERVER': settings.EFS_SERVER,
        'EFS_PATH': settings.EFS_PATH,
        'EFS_EXTEND': settings.EFS_EXTEND,
        'PVS_PATH': settings.PVS_PATH
    }
    k8s_template_file = '%s/update-anchor-peer/fabric-deployment-anchor-peer.yaml' % util.get_k8s_template_path()
    settings.k8s.apply_yaml_from_template(
        namespace=domain, k8s_template_file=k8s_template_file, dict_env=dict_env)

def delete_job_update_anchor_peer(org):
    # Get domain
    domain = util.get_domain(org)
    jobname = 'update-anchor-peer-%s-%s' % (org, settings.CHANNEL_NAME)
    # Delete job pod
    return settings.k8s.delete_job(name=jobname, namespace=domain)

def setup_all():
    orgs = settings.PEER_ORGS.split(' ')
    for org in orgs:
        update_anchor_peer(org)

def del_all_job():
    orgs = settings.PEER_ORGS.split(' ')
    for org in orgs:
        delete_job_update_anchor_peer(org)

@click.group()
def anchor_peer():
    """Anchor Peer config"""
    pass

@anchor_peer.command('setup', short_help="Create job to update anchor peer")
def setup_anchor_peer():
    hiss.rattle('Update anchor peer')
    setup_all()

@anchor_peer.command('delete', short_help="Delete job update anchor peer")
def delete_anchor_peer():
    hiss.rattle('Delete job config anchor peer')
    del_all_job()


import click
import yaml
import re
from kubernetes import client
from os import path
from utils import hiss, util
from settings import settings

def enroll_peer(peer, index):
    # Get domain
    domain = util.get_domain(peer)

    # Get external domain
    external_domain = util.get_peer_external_domain(peer, index)

    # Create temp folder & namespace
    settings.k8s.prereqs(domain)

    k8s_template_file = '%s/enroll-peer/fabric-deployment-enroll-peer.yaml' % util.get_k8s_template_path()
    dict_env = {
        'PEER': peer,
        'PEER_DOMAIN': domain,
        'PEER_INDEX': index,
        'EXTERNAL_PEER_HOST': external_domain,
        'FABRIC_CA_TAG': settings.FABRIC_CA_TAG,
        'EFS_SERVER': settings.EFS_SERVER,
        'EFS_PATH': settings.EFS_PATH,
        'EFS_EXTEND': settings.EFS_EXTEND,
        'PVS_PATH': settings.PVS_PATH
    }

    settings.k8s.apply_yaml_from_template(
        namespace=domain, k8s_template_file=k8s_template_file, dict_env=dict_env)

def del_enroll_peer(peer, index):

    # Get domain
    domain = util.get_domain(peer)
    jobname = 'enroll-p%s-%s' % (index, peer)

    # Delete job pod
    return settings.k8s.delete_job(name=jobname, namespace=domain)

def enroll_all_peer():
    peers = settings.PEER_ORGS.split(' ')
    # TODO: Multiprocess
    for peer in peers:
        for index in range(int(settings.NUM_PEERS)):
            enroll_peer(peer, str(index))

def del_all_enroll_peer():
    peers = settings.PEER_ORGS.split(' ')
    # TODO: Multiprocess
    for peer in peers:
        for index in range(int(settings.NUM_PEERS)):
            del_enroll_peer(peer, str(index))

@click.group()
def enroll_peers():
    """Enroll peers"""
    pass

@enroll_peers.command('setup', short_help="Setup enroll peers")
def setup():
    hiss.rattle('Enroll peers')
    enroll_all_peer()

@enroll_peers.command('delete', short_help="Delete enroll peers")
def delete():
    hiss.rattle('Delete enroll peers job')
    del_all_enroll_peer()



import os
from os.path import expanduser
import click
from settings import settings

from utils import hiss

from k8s.vpn.commands import vpn
from k8s.secret.commands import secret
from k8s.environment.commands import environment
from k8s.config.commands import extract_config

from blockchain.copyscripts.commands import copyscripts
from blockchain.reg_orgs.commands import reg_orgs
from blockchain.reg_orderers.commands import reg_orderers
from blockchain.reg_peers.commands import reg_peers
from blockchain.enroll_orderers.commands import enroll_orderers
from blockchain.enroll_peers.commands import enroll_peers
from blockchain.rca.commands import rca
from blockchain.ica.commands import ica
from blockchain.zookeeper.commands import zookeeper
from blockchain.kafka.commands import kafka
from blockchain.orderer.commands import orderer
from blockchain.update_folder.commands import updatefolder
from blockchain.channel_artifact.commands import channel_artifact
from blockchain.peer.commands import peer
from blockchain.update_anchor_peer.commands import anchor_peer
from blockchain.gen_artifact.commands import gen_artifact
from blockchain.admin.commands import admin
from blockchain.admin_v1.commands import adminv1
from blockchain.bootstrap_network.commands import bootstrap
from blockchain.start_network.commands import start
from blockchain.delete_network.commands import delete
from blockchain.terminate_network.commands import terminate

from blockchain.explorer.commands import explorer
from blockchain.prometheus.commands import prometheus
from blockchain.grafana.commands import grafana

from blockchain.create_org.commands import create_org
from blockchain.update_channel_config.commands import channel_config

from blockchain.external_chaincode.commands import externalCC
from blockchain.generate_ccp.commands import ccp


@click.group()
def mamba():
    # Setup all shared global utilities in settings module
    settings.init()

mamba.add_command(extract_config)
mamba.add_command(environment)
mamba.add_command(vpn)
mamba.add_command(copyscripts)
mamba.add_command(reg_orgs)
mamba.add_command(reg_orderers)
mamba.add_command(reg_peers)
mamba.add_command(enroll_orderers)
mamba.add_command(enroll_peers)
mamba.add_command(rca)
mamba.add_command(ica)
mamba.add_command(zookeeper)
mamba.add_command(kafka)
mamba.add_command(orderer)
mamba.add_command(updatefolder)
mamba.add_command(peer)
mamba.add_command(gen_artifact)
mamba.add_command(channel_artifact)
mamba.add_command(admin)
mamba.add_command(adminv1)
mamba.add_command(secret)
mamba.add_command(bootstrap)
mamba.add_command(start)
mamba.add_command(delete)
mamba.add_command(terminate)
mamba.add_command(explorer)
mamba.add_command(prometheus)
mamba.add_command(grafana)
mamba.add_command(create_org)
mamba.add_command(channel_config)
mamba.add_command(anchor_peer)
mamba.add_command(externalCC)
mamba.add_command(ccp)

if __name__ == '__main__':
    mamba()



import click
import yaml
import re
from kubernetes import client
from os import path
from utils import hiss, util
from settings import settings

def setup_admin(org):

    # Get domain
    domain = util.get_domain(org)

    # Get orderer information
    orderer_names = settings.ORDERER_ORGS.split(' ')
    orderer_domains = settings.ORDERER_DOMAINS.split(' ')
    if orderer_names == '' and settings.REMOTE_ORDERER_NAME != '':
        orderer_names = settings.REMOTE_ORDERER_NAME.split(' ')
        orderer_domains = settings.REMOTE_ORDERER_DOMAIN.split(' ')

    # Build endorsement config
    peer_orgs = '%s %s' % (settings.PEER_ORGS, settings.ENDORSEMENT_ORG_NAME)
    peer_domains = '%s %s' % (settings.PEER_DOMAINS, settings.ENDORSEMENT_ORG_DOMAIN)
    print(peer_orgs)

    # Create application artifact folder
    hiss.echo('Create wallet folder')
    ## Find efs pod
    pods = settings.k8s.find_pod(namespace="default", keyword="test-efs")
    if not pods:
        return hiss.hiss('cannot find tiller pod')

    mkdir_cmd = ('mkdir -p '+settings.EFS_ROOT+'/admin-v2/wallet;')

    # Exec command
    exec_command = [
        '/bin/bash',
        '-c',
        '%s'  % (mkdir_cmd)]

    result_get_folder = settings.k8s.exec_pod(
        podName=pods[0], namespace="default", command=exec_command)
    hiss.sub_echo(result_get_folder.data)

    # Create temp folder & namespace
    settings.k8s.prereqs(domain)
    dict_env = {
        'ORG_NAME': org,
        'ORG_DOMAIN': domain,
        'PEER_NAMES': peer_orgs,
        'PEER_DOMAINS': peer_domains,
        'ORDERER_DOMAIN': orderer_domains[0],
        'ORGDERER_NAME': orderer_names[0],
        'EFS_SERVER': settings.EFS_SERVER,
        'EFS_PATH': settings.EFS_PATH,
        'EFS_EXTEND': settings.EFS_EXTEND,
        'PVS_PATH': settings.PVS_PATH
    }

    # Apply deployment
    k8s_template_file = '%s/admin/admin-deployment.yaml' % util.get_k8s_template_path()
    settings.k8s.apply_yaml_from_template(
        namespace=domain, k8s_template_file=k8s_template_file, dict_env=dict_env)

    # Apply service
    k8s_template_file = '%s/admin/admin-service.yaml' % util.get_k8s_template_path()
    settings.k8s.apply_yaml_from_template(
        namespace=domain, k8s_template_file=k8s_template_file, dict_env=dict_env)

def del_admin(org):

    # Get domain
    domain = util.get_domain(org)
    name = 'admin-v2-%s' % org

    # Delete job pod
    return settings.k8s.delete_stateful(name=name, namespace=domain)

def setup_all_admin():
    orgs = settings.PEER_ORGS.split(' ')
    # TODO: Multiprocess
    for org in orgs:
        setup_admin(org)

def delete_all_admin():
    orgs = settings.PEER_ORGS.split(' ')
    # TODO: Multiprocess
    for org in orgs:
        del_admin(org)

@click.group()
def admin():
    '''Admin Service'''
    pass

@admin.command('setup', short_help='Create new a new Admin service')
def setup():
    hiss.rattle('Create new Admin service in all org')

    setup_all_admin()

@admin.command('delete', short_help='Delete the Admin service')
def delete():
    hiss.rattle('Delete the Admin service in all org')

    delete_all_admin()



import os
import yaml
import time
from kubernetes import client, config
from kubernetes.client.rest import ApiException
from kubernetes.stream import stream
from utils import hiss, util
from k8s.namespace import Namespace
from settings import settings


class KubeHelper:

    config.load_kube_config()
    coreApi = client.CoreV1Api()
    appsApi = client.AppsV1Api()
    batchApi = client.BatchV1Api()

    def read_pod_log(self, name, namespace):
        pods = self.find_pod(namespace, name)
        if not pods:
            msg = 'Pod %s not found!' % name
            return util.Result(success=False, msg=msg)

        for pod in pods:
            try:
                api_response = self.coreApi.read_namespaced_pod_log(
                    pod, namespace)
                print(api_response)
            except ApiException as e:
                print(
                    "Exception when calling CoreV1Api->read_namespaced_pod_log: %s\n" % e)

    def read_stateful_set(self, name, namespace):
        try:
            api_response = self.appsApi.read_namespaced_stateful_set(
                name, namespace)
            return util.Result(success=True, data=api_response)
        except ApiException as e:
            msg = "Exception when calling AppsV1Api->read_namespaced_stateful_set: %s\n" % e
            return util.Result(success=False, msg=msg)

    def delete_job(self, name, namespace):
        try:
            body = client.V1DeleteOptions(propagation_policy='Background')
            api_response = self.batchApi.delete_namespaced_job(
                name, namespace, body=body)
            hiss.echo('Delete job %s on namespace %s success' %
                      (name, namespace))
            return util.Result(success=True, data=api_response)
        except ApiException as e:
            err_msg = "Exception when calling BatchV1Api->delete_namespaced_job: %s\n" % e
            return util.Result(success=hiss.hiss(err_msg), msg=err_msg)

    def delete_persistent_volume_claim(self, name, namespace):
        try:
            api_response = self.coreApi.delete_namespaced_persistent_volume_claim(
                name=name, namespace=namespace)
            return util.Result(success=True, data=api_response)
        except ApiException as e:
            err_msg = "Exception when calling CoreV1Api->delete_namespaced_persistent_volume_claim: %s\n" % e
            return util.Result(success=hiss.hiss(err_msg), msg=err_msg)

    def delete_stateful(self, name, namespace, delete_pvc=False):
        action = 'Delete'
        if delete_pvc == True:
            action = 'Terminate'
            read_result = self.read_stateful_set(name, namespace)
            if read_result.success == False:
                return read_result
            volume_claim_templates = read_result.data.spec.volume_claim_templates
            if volume_claim_templates != None:
                for pvt in volume_claim_templates:
                    pvt_name = pvt.metadata.name
                    list_pvc = self.find_pvc(namespace, keyword=pvt_name)
                    for pvc in list_pvc:
                        hiss.echo('Delete pvc %s ' % pvc)
                        self.delete_persistent_volume_claim(
                            name=pvc, namespace=namespace)

        try:
            body = client.V1DeleteOptions(propagation_policy='Background')
            api_response = self.appsApi.delete_namespaced_stateful_set(
                name, namespace, body=body)
            self.check_pod_status_by_keyword(
                keyword=name, namespace=namespace, is_delete=True)
            hiss.echo('%s stateful set %s on namespace %s success' %
                      (action, name, namespace))
            return util.Result(success=True, data=api_response)
        except ApiException as e:
            err_msg = "Exception when calling AppsV1Api->delete_namespaced_stateful_set: %s\n" % e
            return util.Result(success=hiss.hiss(err_msg), msg=err_msg)

    def check_pod_status_by_keyword(self, keyword, namespace, is_delete=False, check_job_success=False):

        condition_status = 'Running' if is_delete else 'Pending'
        condition_status = 'Succeeded' if check_job_success else condition_status

        # Check status
        count = 0  # Use count variable to detect replica
        msg = ''  # Message show the status of the pod
        while True:
            time.sleep(1)
            # Find efs pod
            pods = self.find_pod(
                namespace=namespace, keyword=keyword)
            if not pods:
                if is_delete:
                    hiss.sub_echo('Done')
                    break
                hiss.sub_echo(
                    'cannot find tiller pod when check status.. retry')
                continue

            if is_delete:
                newMsg = '%s terminating' % keyword
                if msg != newMsg:
                    msg = newMsg
                    hiss.sub_echo(msg)
                time.sleep(1)
                continue

            # Check replication
            if len(pods) == count:
                break
            # Check status
            while True:
                resp = self.coreApi.read_namespaced_pod_status(name=pods[count],
                                                               namespace=namespace)

                # Manage notify display
                newMsg = '%s %s' % (pods[count], resp.status.phase)
                if msg != newMsg:
                    msg = newMsg
                    hiss.sub_echo(msg)

                # hiss.sub_echo('%s %s' % (pods[count], resp.status.phase))
                if check_job_success:
                    if resp.status.phase == condition_status:
                        count += 1
                        break
                    time.sleep(1)
                else:
                    if resp.status.phase != condition_status:
                        count += 1
                        break
                    time.sleep(1)

    def prereqs(self, namespace):
        # Create temp folder
        util.make_temp_folder()

        # Create namespace
        settings.k8s.create_namespace(namespace)

    def remove_pvc(self, doc):
        if "volumeClaimTemplates" in doc['spec']:
            volume_claim_templates = doc['spec']['volumeClaimTemplates']
            for vct in volume_claim_templates:
                vct_name = vct['metadata']['name']

                # Get containers in doc
                containers = doc['spec']['template']['spec']['containers']
                for container in containers:
                    volume_mounts = container['volumeMounts']
                    # Find and remove volume mount
                    volume = next(
                        (volume for volume in volume_mounts if volume['name'] == vct_name), None)
                    volume_mounts.remove(volume)

                # Remove volume claim template
                volume_claim_templates.remove(vct)

    def apply_yaml_from_template(self, namespace, k8s_template_file, dict_env):
        yaml_path, _ = util.load_yaml_config_template(
            k8s_template_file, dict_env)
        hiss.sub_echo('Create %s successfully' % yaml_path)

        # Execute yaml
        # hiss.echo('Apply yaml file')
        stream_file = open(yaml_path, 'r')
        docs = yaml.safe_load_all(stream_file)

        success = True
        for doc in docs:
            try:
                if doc['kind'] == 'Service':
                    self.coreApi.create_namespaced_service(namespace, body=doc)
                    print("Create service successfully!")
                    continue
            except ApiException as e:
                print("Service already deployed!")
                continue
            try:
                if doc['kind'] == 'StatefulSet':
                    if settings.DEPLOYMENT_ENV == 'develop':
                        self.remove_pvc(doc)
                    self.appsApi.create_namespaced_stateful_set(
                        namespace, body=doc)
                    self.check_pod_status_by_keyword(
                        keyword=doc['metadata']['name'], namespace=namespace)
                if doc['kind'] == 'Deployment':
                    self.appsApi.create_namespaced_deployment(
                        namespace, body=doc)
                    self.check_pod_status_by_keyword(
                        keyword=doc['metadata']['name'], namespace=namespace)
                if doc['kind'] == 'Job':
                    self.batchApi.create_namespaced_job(namespace, body=doc)
                    self.check_pod_status_by_keyword(
                        keyword=doc['metadata']['name'], namespace=namespace, check_job_success=True)
                if doc['kind'] == 'ConfigMap':
                    self.coreApi.create_namespaced_config_map(
                        namespace, body=doc)
            except ApiException as e:
                print("Exception when apply yaml: %s\n" % e)
                success = False
                # self.check_pod_status_by_keyword(keyword=doc['metadata']['name'], namespace=namespace)
        return success

    def create_namespace(self, name):
        hiss.echo('Create Namespace %s' % name)
        ns = Namespace(name)
        if not ns.get():
            hiss.sub_echo('Namespace %s does not exist. Creating...' % name)
            ns.create()

    def show_all_pods(self):
        ret = self.coreApi.list_pod_for_all_namespaces(watch=False)
        for i in ret.items:
            print("%s\t%s\t%s" %
                  (i.status.pod_ip, i.metadata.namespace, i.metadata.name))

    def make_port_forward(self, podName, namespace, ports):
        try:
            resp = self.coreApi.connect_get_namespaced_pod_portforward(
                name=podName, namespace=namespace, ports=ports)
            return resp
        except ApiException as e:
            hiss.hiss(
                "Exception when calling CoreV1Api->connect_post_namespaced_pod_portforward: %s\n" % e)

    # Find name of the pod in a namespace with a specific keyword
    def find_pod(self, namespace, keyword):
        mypods = []
        try:
            ret = self.coreApi.list_namespaced_pod(namespace)
            for item in ret.items:
                if keyword in item.metadata.name:
                    mypods.append(item.metadata.name)
        except ApiException as e:
            hiss.hiss("Exception when calling Api: %s\n" % e)
        return mypods

    # Find name of the stateful set in a namespace with a specific keyword
    def find_stateful_set(self, namespace, keyword):
        mypods = []
        try:
            ret = self.appsApi.list_namespaced_stateful_set(namespace)
            for item in ret.items:
                if keyword in item.metadata.name:
                    mypods.append(item.metadata.name)
        except ApiException as e:
            hiss.hiss("Exception when calling Api: %s\n" % e)
        return mypods

    # Find name of the persistent volume claim in a namespace with a specific keyword
    def find_pvc(self, namespace, keyword):
        list_pvc = []
        try:
            ret = self.coreApi.list_namespaced_persistent_volume_claim(
                namespace)
            for item in ret.items:
                if keyword in item.metadata.name:
                    list_pvc.append(item.metadata.name)
        except ApiException as e:
            hiss.hiss("Exception when calling Api: %s\n" % e)
        return list_pvc

    # Requests to exec of Pod
    def exec_pod(self, podName, namespace, command):
        try:
            resp = stream(self.coreApi.connect_get_namespaced_pod_exec,
                          name=podName, namespace=namespace, container="test-pod", stderr=True, stdin=True, stdout=True, command=command)
            # return util.resultDict(success=True, msg='Success', data=resp)
            return util.Result(success=True, msg='Success', data=resp)
        except ApiException as e:
            err_msg = "Exception when calling CoreV1Api->connect_get_namespaced_pod_exec: %s\n" % e
            return util.Result(success=hiss.hiss(err_msg), msg=err_msg)

    def cp_to_pod(self, podName, namespace, source, target):
        if os.path.exists(source):
            sourcePath = source
            if os.path.isdir(sourcePath):
                sourcePath = '%s/.' % source
            cmd = 'kubectl cp %s -n %s %s:%s' % (
                sourcePath, namespace, podName, target)
            copyResult = os.system(cmd)
            if copyResult != 0:
                return hiss.hiss('cannot copy to pod')
        else:
            return hiss.hiss('file/folder \'%s\' does not exists' % source)
        return True

    def create_stateful_set(self, stsName, namespace, replicas, containers, volumes, volumeClaimTemplates):
        api_version = 'apps/v1'
        kind = 'StatefulSet'

        metadata = client.V1ObjectMeta(name=stsName, namespace=namespace)

        # Build spec_selector
        spec_selector_match_labels = dict()
        spec_selector_match_labels['name'] = stsName
        spec_selector_match_labels['namespace'] = namespace
        spec_selector = client.V1LabelSelector(
            match_labels=spec_selector_match_labels)

        # Build spec_template
        spec_template_metadata_labels = dict()
        spec_template_metadata_labels['name'] = stsName
        spec_template_metadata_labels['namespace'] = namespace
        spec_template_metadata = client.V1ObjectMeta(
            labels=spec_template_metadata_labels)
        spec_template_spec = client.V1PodSpec(
            containers=containers, volumes=volumes)
        spec_template = client.V1PodTemplateSpec(
            metadata=spec_template_metadata, spec=spec_template_spec)

        # Build spec
        spec = client.V1StatefulSetSpec(service_name=stsName, replicas=replicas, selector=spec_selector,
                                        template=spec_template, volume_claim_templates=volumeClaimTemplates)

        # Build body
        body = client.V1StatefulSet(
            api_version=api_version, kind=kind, metadata=metadata, spec=spec)

        # Create stateful set
        try:
            api_response = self.appsApi.create_namespaced_stateful_set(
                namespace=namespace, body=body)
            print('api_response: ', api_response)
        except ApiException as e:
            return hiss.hiss("Exception when calling AppsV1Api->create_namespaced_stateful_set: %s\n" % e)



import click
import yaml
import re
import datetime
from kubernetes import client
from os import path
from kubernetes.client.rest import ApiException
from utils import hiss, util
from settings import settings


def terminate_peer(peer, index):
    # Get domain
    domain = util.get_domain(peer)

    # peer service name
    name = 'peer%s-%s' % (index, peer) 

    # Terminate peer stateful set
    res_del_peer =  settings.k8s.delete_stateful(name=name, namespace=domain, delete_pvc=True)

    # couchdb service name
    name = 'couchdb%s-%s' % (index, peer) 

    # Terminate couchdb stateful set
    res_del_db =  settings.k8s.delete_stateful(name=name, namespace=domain, delete_pvc=True)

    if res_del_peer.success == True and res_del_db.success == True:
        hiss.sub_echo('Terminate peer & couchdb success')
    return res_del_db

def delete_peer(peer, index):
    # Get domain
    domain = util.get_domain(peer)

    # peer service name
    name = 'peer%s-%s' % (index, peer) 

    # Delete peer stateful set
    res_del_peer =  settings.k8s.delete_stateful(name=name, namespace=domain)

    # couchdb service name
    name = 'couchdb%s-%s' % (index, peer) 

    # Delete couchdb stateful set
    res_del_db =  settings.k8s.delete_stateful(name=name, namespace=domain)

    if res_del_peer.success == True and res_del_db.success == True:
        hiss.sub_echo('Delete peer & couchdb success')

def setup_peer(peer, index):

    # Get domain
    domain = util.get_domain(peer)
    # Create temp folder & namespace
    settings.k8s.prereqs(domain)

    if settings.K8S_TYPE == 'minikube':
        storage_class = 'standard'
    else:
        storage_class = 'gp2'

    dict_env = {
        'PEER_ORG': peer,
        'PEER_DOMAIN': domain,
        'PEER_INDEX': index,
        'FABRIC_TAG': settings.FABRIC_TAG,
        'COUCHDB_TAG': settings.COUCHDB_TAG,
        'EFS_SERVER': settings.EFS_SERVER,
        'EFS_PATH': settings.EFS_PATH,
        'EFS_EXTEND': settings.EFS_EXTEND,
        'PVS_PATH': settings.PVS_PATH,
        'STORAGE_CLASS': storage_class
    }

    peer_stateful = '%s/peer-sts/peer-stateful.yaml' % util.get_k8s_template_path()
    settings.k8s.apply_yaml_from_template(
        namespace=domain, k8s_template_file=peer_stateful, dict_env=dict_env)

    peer_service = '%s/peer-sts/peer-service-stateful.yaml' % util.get_k8s_template_path()
    settings.k8s.apply_yaml_from_template(
        namespace=domain, k8s_template_file=peer_service, dict_env=dict_env)

    num_peers = int(settings.NUM_PEERS)
    if (num_peers > 2):
        hiss.echo('NUM_PEER > 2: Does not support deployment nlb service')
    if (util.get_peer_external_domain(peer, index) != ''):
        peer_service_nlb = '%s/peer-sts/peer-service-nlb.yaml' % util.get_k8s_template_path()
        settings.k8s.apply_yaml_from_template(
            namespace=domain, k8s_template_file=peer_service_nlb, dict_env=dict_env)

def setup_all_peer():
    peers = settings.PEER_ORGS.split(' ')
    for peer in peers:
        for index in range(int(settings.NUM_PEERS)):
            setup_peer(peer, str(index))

def del_all_peer():
    peers = settings.PEER_ORGS.split(' ')
    for peer in peers:
        for index in range(int(settings.NUM_PEERS)):
            delete_peer(peer, str(index))

def terminate_all_peer():
    peers = settings.PEER_ORGS.split(' ')
    results = []
    for peer in peers:
        for index in range(int(settings.NUM_PEERS)):
            results.append(terminate_peer(peer, str(index)))
    return results

@click.group()
def peer():
    """Peer"""
    pass


@peer.command('setup', short_help="Setup peers")
def setup():
    hiss.rattle('Setup peers')
    setup_all_peer()

@peer.command('delete', short_help="Delete peers")
def delete():
    hiss.rattle('Delete peers')
    del_all_peer()

@peer.command('terminate', short_help="Terminate peers")
def terminate():
    hiss.rattle('Terminate peers')
    terminate_all_peer()



import click
import yaml
import re
import os
from kubernetes import client
from os import path
from utils import hiss, util
from settings import settings

def create_docker_secret(namespace, name):
    create_cmd = ('kubectl create secret docker-registry '+name+
    ' --docker-server='+settings.PRIVATE_DOCKER_SEVER+
    ' --docker-username='+settings.PRIVATE_DOCKER_USER+
    ' --docker-password='+settings.PRIVATE_DOCKER_PASSWORD+
    ' --docker-email='+settings.PRIVATE_DOCKER_EMAIL+
    ' -n '+namespace)
    # print(create_cmd)
    res = os.system(create_cmd)
    return True if res == 0 else False

def create_all_docker_secret(name):
    if settings.ORDERER_DOMAINS == '':
        hiss.sub_echo('Create secret in namespace: default')
        create_docker_secret('default', name)

    domains = settings.DOMAINS.split(' ')
    for domain in domains:
        hiss.sub_echo('Create secret in namespace: %s' % domain)
        create_docker_secret(domain, name)

def delete_secret(namespace, name):
    delete_cmd = ('kubectl delete secret '+name+' -n '+namespace)
    res = os.system(delete_cmd)
    return True if res == 0 else False

def delete_all_docker_secret(name):
    if settings.ORDERER_DOMAINS == '':
        hiss.sub_echo('Delete secret in namespace: default')
        delete_secret('default', name)

    domains = settings.DOMAINS.split(' ')
    for domain in domains:
        hiss.sub_echo('Delete secret in namespace: %s' % domain)
        delete_secret(domain, name)

@click.group()
def secret():
    '''Secret'''
    pass

@secret.command('create', short_help='Create docker secret in all namespace')
def create():
    hiss.rattle('Create docker secret in all namespace')

    create_all_docker_secret('mamba')

@secret.command('delete', short_help='Delete docker secret in all namespace')
def delete():
    hiss.rattle('Delete docker secret in all namespace')

    delete_all_docker_secret('mamba')



import click
import yaml
import re
import datetime
from kubernetes import client
from os import path
from kubernetes.client.rest import ApiException
from utils import hiss, util
from settings import settings


def terminate_ica(ica_org):
    ica_name = 'ica-%s' % ica_org
    # Get domain ica to create namespace
    ica_domain = util.get_domain(ica_org)
    if not ica_domain:
        return hiss.hiss('Fail to get domain of %s ' % ica_org)

    # Terminate stateful set
    return settings.k8s.delete_stateful(name=ica_name, namespace=ica_domain, delete_pvc=True)

def delete_ica(ica_org):
    ica_name = 'ica-%s' % ica_org
    # Get domain ica to create namespace
    ica_domain = util.get_domain(ica_org)
    if not ica_domain:
        return hiss.hiss('Fail to get domain of %s ' % ica_org)

    # Delete stateful set
    return settings.k8s.delete_stateful(name=ica_name, namespace=ica_domain)

def setup_ica(ica_org):

    # Get domain ica to create namespace
    ica_domain = util.get_domain(ica_org)
    if not ica_domain:
        return hiss.hiss('Fail to get domain of %s ' % ica_org)

    # Create temp folder & namespace
    settings.k8s.prereqs(ica_domain)

    ica_name = 'ica-%s' % ica_org

    rca_host = settings.REMOTE_RCA_ADDRESS if settings.RCA_NAME == '' else '%s.%s' % (settings.RCA_NAME, settings.RCA_DOMAIN)

    storage_class = 'standard' if settings.K8S_TYPE == 'minikube' else 'gp2'

    rca_name = settings.RCA_NAME or settings.REMOTE_RCA_NAME

    k8s_template_file = '%s/ica/fabric-deployment-ica.yaml' % util.get_k8s_template_path()
    dict_env = {
        'ORG': ica_org,
        'ICA_NAME': ica_name,
        'ICA_DOMAIN': ica_domain,
        'RCA_NAME': rca_name,
        'RCA_HOST': rca_host,
        'FABRIC_CA_TAG': settings.FABRIC_CA_TAG,
        'EFS_SERVER': settings.EFS_SERVER,
        'EFS_PATH': settings.EFS_PATH,
        'EFS_EXTEND': settings.EFS_EXTEND,
        'PVS_PATH': settings.PVS_PATH,
        'STORAGE_CLASS': storage_class
    }
    settings.k8s.apply_yaml_from_template(
        namespace=ica_domain, k8s_template_file=k8s_template_file, dict_env=dict_env)

def setup_all_ica():
    
    orgs = settings.ORGS.split(' ')
    # TODO: Multiprocess
    for org in orgs:
        setup_ica(org)

def delete_all_ica():
    orgs = settings.ORGS.split(' ')
    # TODO: Multiprocess
    for org in orgs:
        delete_ica(org)

def terminate_all_ica():
    orgs = settings.ORGS.split(' ')
    result = []
    # TODO: Multiprocess
    for org in orgs:
        result.append(terminate_ica(org))
    return result

@click.group()
def ica():
    """Intermediate Certificate Authority"""
    pass


@ica.command('setup', short_help="Setup Intermediate CA")
def setup():
    hiss.rattle('Setup ICA')
    setup_all_ica()

@ica.command('delete', short_help="Delete Intermediate CA")
def delete():
    hiss.rattle('Delete ICA')
    delete_all_ica()

@ica.command('terminate', short_help="Terminate Intermediate CA")
def terminate():
    hiss.rattle('Terminate Intermediate CA Server')
    terminate_all_ica()



import click
import yaml
import re
from kubernetes import client
from os import path
from utils import hiss, util
from settings import settings

def reg_orderer(orderer):
    # Get domain
    domain = util.get_domain(orderer)

    # Create temp folder & namespace
    settings.k8s.prereqs(domain)

    k8s_template_file = '%s/register-orderer/fabric-deployment-register-orderer.yaml' % util.get_k8s_template_path()
    dict_env = {
        'ORDERER_ORG': orderer,
        'ORDERER_DOMAIN': domain,
        'FABRIC_CA_TAG': settings.FABRIC_CA_TAG,
        'EFS_SERVER': settings.EFS_SERVER,
        'EFS_PATH': settings.EFS_PATH,
        'EFS_EXTEND': settings.EFS_EXTEND,
        'PVS_PATH': settings.PVS_PATH
    }

    settings.k8s.apply_yaml_from_template(
        namespace=domain, k8s_template_file=k8s_template_file, dict_env=dict_env)

def del_reg_orderer(orderer):

    # Get domain
    domain = util.get_domain(orderer)
    jobname = 'register-o-%s' % orderer

    # Delete job pod
    return settings.k8s.delete_job(name=jobname, namespace=domain)

def reg_all_orderer():
    orderers = settings.ORDERER_ORGS.split(' ')
    # TODO: Multiprocess
    for orderer in orderers:
        reg_orderer(orderer)

def del_all_reg_orderer():
    orderers = settings.ORDERER_ORGS.split(' ')
    # TODO: Multiprocess
    for orderer in orderers:
        del_reg_orderer(orderer)

@click.group()
def reg_orderers():
    """Register orderers"""
    pass

@reg_orderers.command('setup', short_help="Setup register orderers")
def setup():
    hiss.rattle('Register orderers')
    reg_all_orderer()

@reg_orderers.command('delete', short_help="Delete register orderers")
def delete():
    hiss.rattle('Delete register orderers job')
    del_all_reg_orderer()



import click
import yaml
import re
import datetime
from kubernetes import client
from os import path
from kubernetes.client.rest import ApiException
from utils import hiss, util
from settings import settings


def terminate_orderer(orderer, index):
    # Get domain
    domain = util.get_domain(orderer)

    name = 'orderer%s-%s' % (index, orderer) 

    # Terminate stateful set
    return settings.k8s.delete_stateful(name=name, namespace=domain, delete_pvc=True)

def delete_orderer(orderer, index):
    # Get domain
    domain = util.get_domain(orderer)

    name = 'orderer%s-%s' % (index, orderer) 

    # Delete stateful set
    return settings.k8s.delete_stateful(name=name, namespace=domain)

def setup_orderer(orderer, index):

    # Get domain
    domain = util.get_domain(orderer)
    # Create temp folder & namespace
    settings.k8s.prereqs(domain)

    if settings.K8S_TYPE == 'minikube':
        store_class = 'standard'
    else:
        store_class = 'gp2'

    dict_env = {
        'ORDERER': orderer,
        'ORDERER_DOMAIN': domain,
        'ORDERER_INDEX': index,
        'FABRIC_TAG': settings.FABRIC_TAG,
        'EFS_SERVER': settings.EFS_SERVER,
        'EFS_PATH': settings.EFS_PATH,
        'EFS_EXTEND': settings.EFS_EXTEND,
        'PVS_PATH': settings.PVS_PATH,
        'STORE_CLASS': store_class
    }

    orderer_stateful = '%s/orderer-sts/orderer-stateful.yaml' % util.get_k8s_template_path()
    settings.k8s.apply_yaml_from_template(
        namespace=domain, k8s_template_file=orderer_stateful, dict_env=dict_env)

    orderer_service = '%s/orderer-sts/orderer-service.yaml' % util.get_k8s_template_path()
    settings.k8s.apply_yaml_from_template(
        namespace=domain, k8s_template_file=orderer_service, dict_env=dict_env)

    if settings.EXTERNAL_ORDERER_ADDRESSES != '':
        # Setup nlb
        orderer_service_nlb = '%s/orderer-sts/orderer-service-nlb.yaml' % util.get_k8s_template_path()
        settings.k8s.apply_yaml_from_template(
            namespace=domain, k8s_template_file=orderer_service_nlb, dict_env=dict_env)

def setup_all_orderer():
    orderers = settings.ORDERER_ORGS.split(' ')
    for orderer in orderers:
        for index in range(int(settings.NUM_ORDERERS)):
            setup_orderer(orderer, str(index))

def delete_all_orderer():
    orderers = settings.ORDERER_ORGS.split(' ')
    for orderer in orderers:
        for index in range(int(settings.NUM_ORDERERS)):
            delete_orderer(orderer, str(index))

def terminate_all_orderer():
    orderers = settings.ORDERER_ORGS.split(' ')
    results = []
    for orderer in orderers:
        for index in range(int(settings.NUM_ORDERERS)):
            results.append(terminate_orderer(orderer, str(index)))
    return results

@click.group()
def orderer():
    """Orderer"""
    pass


@orderer.command('setup', short_help="Setup Orderers")
def setup():
    hiss.rattle('Setup orderers')
    setup_all_orderer()

@orderer.command('delete', short_help="Delete Orderer")
def delete():
    hiss.rattle('Delete Orderer')
    delete_all_orderer()

@orderer.command('terminate', short_help="Terminate Orderer")
def terminate():
    hiss.rattle('Terminate Orderer')
    terminate_all_orderer()



import click
import yaml
import re
from kubernetes import client
from os import path
from utils import hiss, util
from settings import settings

def terminate_rca():
    name = settings.RCA_NAME
    domain = settings.RCA_DOMAIN

    # Terminate stateful set
    return settings.k8s.delete_stateful(name=name, namespace=domain, delete_pvc=True) 


def delete_rca():
    name = settings.RCA_NAME
    domain = settings.RCA_DOMAIN

    # Delete stateful set
    return settings.k8s.delete_stateful(name=name, namespace=domain)


def setup_rca():
    domain = settings.RCA_DOMAIN

    # Create temp folder & namespace
    settings.k8s.prereqs(domain)

    if settings.K8S_TYPE == 'minikube':
        storage_class = 'standard'
    else:
        storage_class = 'gp2'

    dict_env = {
        'ORG': domain,
        'RCA_NAME': settings.RCA_NAME,
        'FABRIC_ORGS': settings.ORGS,
        'FABRIC_CA_TAG': settings.FABRIC_CA_TAG,
        'EFS_SERVER': settings.EFS_SERVER,
        'EFS_PATH': settings.EFS_PATH,
        'EFS_EXTEND': settings.EFS_EXTEND,
        'PVS_PATH': settings.PVS_PATH,
        'STORAGE_CLASS': storage_class
    }

    k8s_template_file = '%s/rca/fabric-deployment-rca.yaml' % util.get_k8s_template_path()
    settings.k8s.apply_yaml_from_template(
        namespace=domain, k8s_template_file=k8s_template_file, dict_env=dict_env)

    if settings.EXTERNAL_RCA_ADDRESSES != '':
        # Deploy nlb
        k8s_nlb_template_file = '%s/rca/fabric-deployment-rca-nlb.yaml' % util.get_k8s_template_path()
        settings.k8s.apply_yaml_from_template(
        namespace=domain, k8s_template_file=k8s_nlb_template_file, dict_env=dict_env)



@click.group()
def rca():
    """Root Certificate Authority"""
    pass


@rca.command('setup', short_help="Setup Root CA")
def setup():
    hiss.rattle('Setup Root CA Server')
    setup_rca()

@rca.command('delete', short_help="Delete Root CA")
def delete():
    hiss.rattle('Delete Root CA Server')
    delete_rca()

@rca.command('terminate', short_help="Terminate Root CA")
def terminate():
    hiss.rattle('Terminate Root CA Server')
    terminate_rca()


from os import path
import click
import yaml
import re
from kubernetes import client
from utils import hiss, util
from settings import settings

def bootstrap_network():

    orderer_orgs = settings.ORDERER_ORGS.split(' ')
    orderer_domains = settings.ORDERER_DOMAINS.split(' ')

    peer_orgs = settings.PEER_ORGS.split(' ')
    peer_domains = settings.PEER_DOMAINS.split(' ')


    # Create temp folder & namespace
    settings.k8s.prereqs(orderer_domains[0])

    k8s_template_file = '%s/bootstrap-network/fabric-deployment-bootstrap-network.yaml' % util.get_k8s_template_path()
    dict_env = {
        'ORDERER_NAME': orderer_orgs[0],
        'ORDERER_DOMAIN': orderer_domains[0],
        'ORG_NAME': peer_orgs[0],
        'ORG_DOMAIN': peer_domains[0],
        'EFS_SERVER': settings.EFS_SERVER,
        'EFS_PATH': settings.EFS_PATH,
        'EFS_EXTEND': settings.EFS_EXTEND,
        'PVS_PATH': settings.PVS_PATH
    }

    settings.k8s.apply_yaml_from_template(
        namespace=orderer_domains[0], k8s_template_file=k8s_template_file, dict_env=dict_env)

def del_bootstrap_network():

    domains = settings.ORDERER_DOMAINS.split(' ')
    jobname = 'bootstrap-network'

    # Delete job pod
    return settings.k8s.delete_job(name=jobname, namespace=domains[0])
    

@click.group()
def bootstrap():
    """Bootstrap network"""
    pass

@bootstrap.command('setup', short_help="Run job to bootstrap network")
def setup():
    hiss.rattle('Generate channel.tx, genesis.block')

    bootstrap_network()

@bootstrap.command('delete', short_help="Delete job bootstrap network")
def delete():
    hiss.rattle('Delete job channel artifact')

    del_bootstrap_network()



import setuptools

with open("README.md", "r") as fh:
    LONG_DESCRIPTION = fh.read()

with open('requirements.txt', 'r') as f:
    REQUIREMENTS = f.read().splitlines()
    print(REQUIREMENTS)

setuptools.setup(
    name='akc-mamba',
    version='2.1.1a1',
    entry_points={'console_scripts': ['mamba = cli.mamba:mamba']},
    author="akaChain",
    author_email="admin@akachain.io",
    description="A production ready, complete experience in deploying a Hyperledger Fabric",
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    url="https://github.com/Akachain/akc-mamba",
    package_data={'': ['*']},
    include_package_data=True,
    install_requires=REQUIREMENTS,
    # package_dir={'': 'mamba'},
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    keywords='mamba akc-mamba hyperledger fabric blockchain network',
)



import click
import yaml
import re
from kubernetes import client
from os import path
from utils import hiss, util
from settings import settings

def reg_org(org):
    # Get domain
    domain = util.get_domain(org)

    # Create temp folder & namespace
    settings.k8s.prereqs(domain)

    k8s_template_file = '%s/register-org/fabric-deployment-register-org.yaml' % util.get_k8s_template_path()
    dict_env = {
        'ORG': org,
        'REG_DOMAIN': domain,
        'FABRIC_CA_TAG': settings.FABRIC_CA_TAG,
        'EFS_SERVER': settings.EFS_SERVER,
        'EFS_PATH': settings.EFS_PATH,
        'EFS_EXTEND': settings.EFS_EXTEND,
        'PVS_PATH': settings.PVS_PATH
    }

    settings.k8s.apply_yaml_from_template(
        namespace=domain, k8s_template_file=k8s_template_file, dict_env=dict_env)

def del_reg_org(org):

    # Get domain
    domain = util.get_domain(org)
    jobname = 'register-org-%s' % org

    # Delete job pod
    return settings.k8s.delete_job(name=jobname, namespace=domain)

def reg_all_org():
    orgs = settings.ORGS.split(' ')
    # TODO: Multiprocess
    for org in orgs:
        reg_org(org)

def del_all_reg_org():
    orgs = settings.ORGS.split(' ')
    # TODO: Multiprocess
    for org in orgs:
        del_reg_org(org)

@click.group()
def reg_orgs():
    """Register organizations"""
    pass

@reg_orgs.command('setup', short_help="Setup register organizations")
def setup():
    hiss.rattle('Register organizations')
    reg_all_org()


@reg_orgs.command('delete', short_help="Delete register organizations")
def delete():
    hiss.rattle('Delete register organizations job')
    del_all_reg_org()



import click
import yaml
import re
import json
import os
import time
from kubernetes import client
from os import path
from utils import hiss, util
from settings import settings

def get_namespace():
    # Get domain
    domains = settings.ORDERER_DOMAINS.split(' ')
    if len(domains) == 0:
        domains = settings.PEER_DOMAINS.split(' ')
    explorer_namespace = domains[0]
    
    # Create temp folder & namespace
    settings.k8s.prereqs(explorer_namespace)
    return explorer_namespace

def setup_explorer_db():
    # Get domain
    explorer_db_namespace = get_namespace()

    # Create temp folder & namespace
    settings.k8s.prereqs(explorer_db_namespace)

    dict_env = {
        'DOMAIN': explorer_db_namespace,
        'DATABASE_PASSWORD': 'Akachain'
    }

    # Deploy explorer db sts
    explorer_db_template = '%s/explorer/explorer-db-deployment.yaml' % util.get_k8s_template_path()
    settings.k8s.apply_yaml_from_template(
        namespace=explorer_db_namespace, k8s_template_file=explorer_db_template, dict_env=dict_env)

    # Deploy explorer db svc
    explorer_db_svc_template = '%s/explorer/explorer-db-service.yaml' % util.get_k8s_template_path()
    settings.k8s.apply_yaml_from_template(
        namespace=explorer_db_namespace, k8s_template_file=explorer_db_svc_template, dict_env=dict_env)

    # Create tables
    ## Find explorer_db pod
    pods = settings.k8s.find_pod(namespace=explorer_db_namespace, keyword="explorer-db")
    if not pods:
        return hiss.hiss('cannot find tiller pod')

    create_tbl_cmd = 'chmod 700 /opt/createdb_new.sh; /opt/createdb_new.sh'
    exec_command = [
        '/bin/bash',
        '-c',
        '%s'  % (create_tbl_cmd)]

    result_get_folder = settings.k8s.exec_pod(
        podName=pods[0], namespace=explorer_db_namespace, command=exec_command)
    hiss.sub_echo(result_get_folder.data)

def generate_explorer_config():
    # Load template config
    config_template_path = os.path.abspath(os.path.join(__file__, "../config.json"))
    with open(config_template_path, 'r') as f:
        explorer_config = json.load(f)

    # Update config
    orgs = settings.PEER_ORGS.split(' ')
    orgs_msp = []
    for org in orgs:
        orgs_msp.append('%sMSP' % org)

    client = {
        orgs[0]: {
            'tlsEnable': True,
            'organization': orgs_msp[0],
            'channel': settings.CHANNEL_NAME,
            'credentialStore': {
                'path': '/opt/explorer/crypto-path/fabric-client-kv-%s' % orgs[0],
                'cryptoStore': {
                    'path': '/tmp/crypto-store/fabric-client-kv-%s' % orgs[0]
                }
            }
        }
    }
    explorer_config['network-configs']['network-1']['clients'] = client

    channel_peers = {}
    for x in range(len(orgs)):
        domain = util.get_domain(orgs[x])
        for y in range(int(settings.NUM_PEERS)):
            peer_name = 'peer%s-%s.%s' % (y, orgs[x], domain)
            channel_peers[peer_name] = {}
    explorer_config['network-configs']['network-1']['channels'] = {
        '%s' % settings.CHANNEL_NAME: {
            'peers': channel_peers,
            'connection': {
                'timeout': {
                    'peer': {
                        "endorser": "6000",
                        "eventReg": "6000",
                        "eventHub": "6000"
                    }
                }
            }
        }
    }

    orgs_config = {}
    # org
    for i in range(len(orgs)):
        domain = util.get_domain(orgs[i])
        p_config = {
            'mspid': '%s' % orgs_msp[i],
            'fullpath': False,
            'adminPrivateKey': {
                'path': '/opt/explorer/crypto-config/peerOrganizations/%s/users/admin/msp/keystore' % domain
            },
            'signedCert': {
                'path': '/opt/explorer/crypto-config/peerOrganizations/%s/users/admin/msp/signcerts' % domain
            }
        }
        orgs_config[orgs_msp[i]] = p_config
    # orderer
    orderers = settings.ORDERER_ORGS.split(' ')
    domain = util.get_domain(orderers[0])
    o_config = {
        'mspid': '%sMSP' % orderers[0],
        'adminPrivateKey': {
            'path': '/opt/explorer/crypto-config/ordererOrganizations/'+domain+'/users/admin/msp/keystore'
        }
    }
    orgs_config['%sMSP' % settings.ORDERER_ORGS] = o_config
    explorer_config['network-configs']['network-1']['organizations'] = orgs_config
    
    # peers
    peers_config = {}
    for x in range(len(orgs)):
        domain = util.get_domain(orgs[x])
        for y in range(int(settings.NUM_PEERS)):
            peer_name = 'peer%s-%s.%s' % (y, orgs[x], domain)
            config = {
                'tlsCACerts': {
                    'path': '/opt/explorer/crypto-config/peerOrganizations/'+domain+'/peers/peer%s.%s' % (y, domain)+'/tls/tlsca.mambatest-cert.pem'
                },
                'url':  'grpcs://%s:7051' % peer_name,
                'eventUrl': 'grpcs://%s:7053' % peer_name,
                'grpcOptions': {
                    'ssl-target-name-override': peer_name
                }
            }
            peers_config[peer_name] = config
    explorer_config['network-configs']['network-1']['peers'] = peers_config
    # orderers
    orderers_config = {}
    for x in range(len(orderers)):
        domain = util.get_domain(orderers[x])
        for y in range(int(settings.NUM_ORDERERS)):
            orderer_name = 'orderer%s-%s.%s' % (y, orderers[y], domain)
            config = {
                'url': 'grpcs://%s:7050' % orderer_name,
                'grpcOptions': {
                    'ssl-target-name-override': orderer_name
                },
                'tlsCACerts': {
                    'path': '/opt/explorer/crypto-config/ordererOrganizations/'+domain+'/orderers/orderer%s.%s' % (y, domain)+'/tls/tlsca.ordererhai-cert.pem'
                }
            }
            orderers_config[orderer_name] = config
    explorer_config['network-configs']['network-1']['orderers'] = orderers_config

    return json.dumps(explorer_config)

def create_explorer_config_in_efs(explorer_config):

    # Find efs pod
    pods = settings.k8s.find_pod(namespace="default", keyword="test-efs")
    if not pods:
        return hiss.hiss('cannot find tiller pod')

    config_path = '%s/explorer-config' % settings.EFS_ROOT
    exec_command = [
        '/bin/bash',
        '-c',
        'mkdir -p '+config_path+'; cd '+config_path+'; echo '+explorer_config+' > config.json']

    create_file = settings.k8s.exec_pod(
        podName=pods[0], namespace="default", command=exec_command)
    if create_file.success == False:
        return hiss.hiss('cannot create explorer config in %s' % pods[0])

def setup_explorer():

    hiss.echo('Generate explorer config')
    config = generate_explorer_config()
    create_explorer_config_in_efs(json.dumps(config))

    hiss.echo('Deploy explorer')
    # Get domain
    explorer_namespace = get_namespace()
    
    # Create temp folder & namespace
    settings.k8s.prereqs(explorer_namespace)

    dict_env = {
        'DOMAIN': explorer_namespace,
        'DATABASE_PASSWORD': 'Akachain',
        'EFS_SERVER': settings.EFS_SERVER,
        'EFS_PATH': settings.EFS_PATH,
        'EFS_EXTEND': settings.EFS_EXTEND,
        'PVS_PATH': settings.PVS_PATH
    }

    # Deploy explorer db sts
    explorer_template = '%s/explorer/explorer-deployment.yaml' % util.get_k8s_template_path()
    settings.k8s.apply_yaml_from_template(
        namespace=explorer_namespace, k8s_template_file=explorer_template, dict_env=dict_env)

    # Deploy explorer db svc
    explorer_svc_template = '%s/explorer/explorer-service.yaml' % util.get_k8s_template_path()
    settings.k8s.apply_yaml_from_template(
        namespace=explorer_namespace, k8s_template_file=explorer_svc_template, dict_env=dict_env)


def del_explorer_db():
    # Delete sts
    return settings.k8s.delete_stateful(name='explorer-db', namespace=get_namespace(), delete_pvc=True)

def del_explorer():
    # Delete sts
    return settings.k8s.delete_stateful(name='explorerpod', namespace=get_namespace())

@click.group()
def explorer():
    """Explorer"""
    pass

@explorer.command('setup', short_help="Setup explorer")
def setup():
    hiss.rattle('Setup explorer DB')
    setup_explorer_db()

    hiss.rattle('Setup explorer')
    setup_explorer()

@explorer.command('delete', short_help="Delete explorer")
def delete():
    hiss.rattle('Delete explorer DB')
    del_explorer_db()

    hiss.rattle('Delete explorer')
    del_explorer()



import click
import yaml
import re
from kubernetes import client
from os import path
from utils import hiss, util
from settings import settings

def setup_admin():

    domains = settings.ORDERER_DOMAINS.split(' ')
    if len(domains) == 0:
        domains = settings.PEER_DOMAINS.split(' ')

    # Create application artifact folder
    hiss.echo('Create application artifact folder')
    ## Find efs pod
    pods = settings.k8s.find_pod(namespace="default", keyword="test-efs")
    if not pods:
        return hiss.hiss('cannot find tiller pod')

    mkdir_cmd = ('mkdir -p '+settings.EFS_ROOT+'/admin/artifacts;'
    'mkdir -p '+settings.EFS_ROOT+'/admin/crypto-path;'
    'mkdir -p '+settings.EFS_ROOT+'/admin/crypto-store;')

    ## Exec command
    exec_command = [
        '/bin/bash',
        '-c',
        '%s'  % (mkdir_cmd)]

    result_get_folder = settings.k8s.exec_pod(
        podName=pods[0], namespace="default", command=exec_command)
    hiss.sub_echo(result_get_folder.data)

    # Create temp folder & namespace
    settings.k8s.prereqs(domains[0])
    dict_env = {
        'ORDERER_DOMAIN': domains[0],
        'EFS_SERVER': settings.EFS_SERVER,
        'EFS_PATH': settings.EFS_PATH,
        'EFS_EXTEND': settings.EFS_EXTEND,
        'PVS_PATH': settings.PVS_PATH
    }

    # Apply deployment
    k8s_template_file = '%s/admin-v1/admin-deployment.yaml' % util.get_k8s_template_path()
    settings.k8s.apply_yaml_from_template(
        namespace=domains[0], k8s_template_file=k8s_template_file, dict_env=dict_env)

    # Apply service
    k8s_template_file = '%s/admin-v1/admin-service.yaml' % util.get_k8s_template_path()
    settings.k8s.apply_yaml_from_template(
        namespace=domains[0], k8s_template_file=k8s_template_file, dict_env=dict_env)

def del_admin():

    domains = settings.ORDERER_DOMAINS.split(' ')
    jobname = 'admin-v1'

    # Delete job pod
    return settings.k8s.delete_stateful(name=jobname, namespace=domains[0])
    

@click.group()
def adminv1():
    '''Admin-V1 Service'''
    pass

@adminv1.command('setup', short_help='Create new a new Admin service')
def setup():
    hiss.rattle('Create new a new Admin-V1 service')

    setup_admin()

@adminv1.command('delete', short_help='Delete the Admin service')
def delete():
    hiss.rattle('Delete the Admin-V1 service')

    del_admin()


import click
import os
import time
from settings import settings
from os import path
from shutil import copyfile
from utils import hiss, util

from blockchain.bootstrap_network.commands import del_bootstrap_network
from blockchain.generate_ccp.commands import delete_all_ccp
from blockchain.channel_artifact.commands import del_gen_channel_artifact
from blockchain.gen_artifact.commands import del_generate_artifact
from blockchain.reg_orgs.commands import del_all_reg_org
from blockchain.reg_orderers.commands import del_all_reg_orderer
from blockchain.reg_peers.commands import del_all_reg_peer
from blockchain.enroll_orderers.commands import del_all_enroll_orderer
from blockchain.enroll_peers.commands import del_all_enroll_peer

from blockchain.rca.commands import terminate_rca
from blockchain.ica.commands import terminate_all_ica
from blockchain.zookeeper.commands import terminate_zookeeper
from blockchain.kafka.commands import terminate_kafka
from blockchain.orderer.commands import terminate_all_orderer
from blockchain.peer.commands import terminate_all_peer
from blockchain.admin.commands import delete_all_admin
from blockchain.external_chaincode.commands import delete_all_external_cc
from blockchain.update_anchor_peer.commands import del_all_job

def remove_cert():
    ## Find explorer_db pod
    pods = settings.k8s.find_pod(namespace='default', keyword="test-efs")
    if not pods:
        return hiss.hiss('cannot find tiller pod')
    
    remove_cert = 'rm -rf %s/*' % (settings.EFS_ROOT)
    exec_command = [
        '/bin/bash',
        '-c',
        '%s'  % (remove_cert)]

    return settings.k8s.exec_pod(
        podName=pods[0], namespace='default', command=exec_command)
    

def terminate_network():

    result = []
    # Delete job
    util.smart_append(result, delete_all_admin())
    util.smart_append(result, del_bootstrap_network())
    util.smart_append(result, delete_all_ccp())
    util.smart_append(result, del_gen_channel_artifact())
    util.smart_append(result, del_generate_artifact())
    util.smart_append(result, del_all_reg_org())
    util.smart_append(result, del_all_reg_orderer())
    util.smart_append(result, del_all_reg_peer())
    util.smart_append(result, del_all_enroll_orderer())
    util.smart_append(result, del_all_enroll_peer())
    util.smart_append(result, del_all_job())

    # Terminate Root Certificate Authority service
    util.smart_append(result, terminate_rca())

    # Terminate Intermediate Certificate Authority services
    util.smart_append(result, terminate_all_ica())

    if settings.ORDERER_TYPE == 'kafka':
        # Terminate Zookeeper services
        util.smart_append(result, terminate_zookeeper())
        # Terminate Kafka services
        util.smart_append(result, terminate_kafka())
    
    # Terminate StatefullSet orderers
    util.smart_append(result, terminate_all_orderer())

    # Terminate StatefullSet peers
    util.smart_append(result, terminate_all_peer())

    # Terminate External chaincode
    util.smart_append(result, delete_all_external_cc())

    # Remove old cert
    util.smart_append(result, remove_cert())

    #TODO: Delete secret

    for r in result: print(r)
    return result


@click.command('terminate', short_help="Terminate Blockchain Network")
def terminate():
    hiss.rattle('Terminate Network')
    terminate_network()



import click
import yaml
import re
from kubernetes import client
from os import path
from utils import hiss, util
from settings import settings

def generate_ccp(org):

    # Get domain
    domain = util.get_domain(org)

    # Create temp folder & namespace
    settings.k8s.prereqs(domain)

    k8s_template_file = '%s/connection-profile/generate-ccp-job.yaml' % util.get_k8s_template_path()
    dict_env = {
        'ORG_NAME': org,
        'ORG_DOMAIN': domain,
        'EFS_SERVER': settings.EFS_SERVER,
        'EFS_PATH': settings.EFS_PATH,
        'EFS_EXTEND': settings.EFS_EXTEND,
        'PVS_PATH': settings.PVS_PATH
    }

    settings.k8s.apply_yaml_from_template(
        namespace=domain, k8s_template_file=k8s_template_file, dict_env=dict_env)

def del_generate_ccp(org):

    # Get domain
    domain = util.get_domain(org)
    jobname = 'generate-ccp-%s' % org

    # Delete job pod
    return settings.k8s.delete_job(name=jobname, namespace=domain)
    
def generate_all_ccp():
    orgs = settings.PEER_ORGS.split(' ')
    # TODO: Multiprocess
    for org in orgs:
        generate_ccp(org)

def delete_all_ccp():
    orgs = settings.PEER_ORGS.split(' ')
    # TODO: Multiprocess
    for org in orgs:
        del_generate_ccp(org)

@click.group()
def ccp():
    """Generate connection profiles"""
    pass

@ccp.command('generate', short_help="Run job to generate connection profiles")
def generate():
    hiss.rattle('Generate connection profiles')

    generate_all_ccp()

@ccp.command('delete', short_help="Delete job generate connection profiles")
def delete():
    hiss.rattle('Delete job generate connection profiles')

    delete_all_ccp()



import click
import os
from os.path import expanduser
from settings import settings
from utils import hiss


def env():
    hiss.rattle('Create environment for network')
    excute = expanduser('~/.akachain/akc-mamba/mamba/k8s/environment/setup.sh')
    os.system(excute)
    return True


@click.command('environment', short_help="Create environment for network")
def environment():
    env()
